﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Threading;

public class DelegateCommand<T> : ICommand
{
    // Fields
    private readonly Func<T, bool> canExecuteMethod;
    private readonly Action<T> executeMethod;

    // Events
    public event EventHandler CanExecuteChanged;

    public event EventHandler IsActiveChanged;

    // Methods
    public DelegateCommand(Action<T> executeMethod) : this(executeMethod, null)
    {
    }

    public DelegateCommand(Action<T> executeMethod, Func<T, bool> canExecuteMethod)
    {
        this.executeMethod = null;
        this.canExecuteMethod = null;
        if ((executeMethod == null) && (canExecuteMethod == null))
        {
            throw new ArgumentNullException("executeMethod");
        }
        this.executeMethod = executeMethod;
        this.canExecuteMethod = canExecuteMethod;
    }

    public bool CanExecute(T parameter)
    {
        return ((this.canExecuteMethod == null) || this.canExecuteMethod(parameter));
    }

    public void Execute(T parameter)
    {
        if (this.executeMethod != null)
        {
            this.executeMethod(parameter);
        }
    }

    protected virtual void OnCanExecuteChanged()
    {
        Dispatcher dispatcher = null;
        if (Deployment.Current != null)
        {
            dispatcher = Deployment.Current.Dispatcher;
        }
        EventHandler canExecuteChanged = this.CanExecuteChanged;
        if (canExecuteChanged != null)
        {
            if (!((dispatcher == null) || dispatcher.CheckAccess()))
            {
                dispatcher.BeginInvoke(new Action(this.OnCanExecuteChanged));
            }
            else
            {
                canExecuteChanged(this, EventArgs.Empty);
            }
        }
    }

    public void RaiseCanExecuteChanged()
    {
        this.OnCanExecuteChanged();
    }

    bool ICommand.CanExecute(object parameter)
    {
        return this.CanExecute((T) parameter);
    }

    void ICommand.Execute(object parameter)
    {
        this.Execute((T) parameter);
    }
}